"""
Provides `pkJson` version information.
"""
# This file is auto-generated! Do not edit!
# Use `seVersion pkJson` to change this file.

from seVersion import Version
__version__ = Version("pkJson", 2024, 1, 0, 0, release_candidate=2)
__setuptools__ = __version__.public()
__all__ = ["__version__", "__setuptools__"]
